/** Put here CSS that should apply to EDIT SCREENS ONLY. **/

#header {
    background-color: #edc06b;
}

/* Breadcrumbs */

.breadcrumbs {
    color: #333;
    padding-bottom: 1em;
}

.breadcrumbs .you-are-here {
    margin-right: 0.5em;
}

.breadcrumbs a {
    text-decoration: none;
    color: #333;
}

.breadcrumbs a:visited {
    color: #333;
}

.breadcrumbs .dropdown-trigger {
    background-color:  #eee;
    border: 1px solid #aaa;
    padding: 0.1em 0.5em;
    z-index: 5;
}

.breadcrumbs .dropdown {
    display: none;
    position: absolute;
    z-index: 1;
    border: 1px solid #aaa;
    background-color: #eee;

	opacity: .95;
	filter: alpha(opacity=95);
	-ms-filter:"progid:DXImageTransform.Microsoft.Alpha(Opacity=95)";
}

.breadcrumbs .dropdown ul {
    margin: 0;
    padding: 0;
    list-style: none;
}

.breadcrumbs .dropdown h4 {
    margin: 0 0.5em 0 0.5em;
    padding: 0.2em;
}

.breadcrumbs .dropdown a {
    display: block;
    padding: 0.2em 1em;
    color: black;
}

.breadcrumbs .dropdown a:hover {
    background-color: white;
}

/* Messages */

.message {
    background-color: #bbb;
    padding: 0.5em 1em;
    margin: 0.5em 2em 1em;
    text-align: center;
    font-weight: bold;
}

.message.success {
    background-color: #b5cc7a;
}

.message.info {
    background-color: #edc06b;
}

.message.error {
    background-color: #ff614e;
}

/* Form styles */

div.form {
    width: 35em;
}

form ul {
    margin: 0;
    padding: 0;
    list-style-stype: none;
}

form li {
	display: block;
    padding: 1em;
}

form li.error {
    padding: 1em;
}

form label.desc {
    display: block;
    margin: 0 0 0.5em 0;
    font-weight: bold;
}

form {
    padding: 1em;
    margin: 1em 1em 2em 0;
    background-color:  #eee;
    border: 1px solid #aaa;
}

form h2 {
    margin-top: 0;
}

form fieldset {
    margin: 0;
    padding: 0;
    border-width: 0;
}

.inline{
	margin: 0;
	padding: 0;
}

.errorLi {
    display: none;
}

form li.error {
    background-color: #ff614e;
}
